----------------------------------------
-- 
----------------------------------------
on Base64Encode (str)
  
  -- if zlibXtra v0.9 or newer (http://dasdeck.de/staff/valentin/xtras/zlib/win/) available (much faster!):
  -- return zx_base64encode_string (str)
  
  
  -- otherwise use (platform-specific) lingo version:
  if the platform contains "win" then 
    return Base64EncodePC (str)
  else 
    return Base64EncodeMAC (str)
  end if
end

----------------------------------------
-- optimized for mac by Alex da Franca
----------------------------------------
on Base64EncodeMAC (aBinaryString, aUrlSafe)
  
  if voidP(aUrlSafe) then aUrlSafe=FALSE
  
  -- Define the lookup table according to whether we are using "URL safe" mode or RFC 2045-compliant mode:
  if( aUrlSafe) then
    vBase64Lookup = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-_"
    vTrail = "."
  else
    vBase64Lookup = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/"
    vTrail = "="
  end if
  
  -- alex:
  subLengthLim = 2000 -- tune for optimization. 2000 (=> chunks of 6000 chars => 3*2000) seems to yield good results
  
  -- Initialise everything else...
  crlf = RETURN & numtochar(10)
  vOutput = ""
  vIndex = 1
  vLengthWhole = aBinaryString.length
  vTrailingBytes = vLengthWhole mod 3
  vLengthWhole = vLengthWhole - (vTrailingBytes)
  
  subLengthLim = subLengthLim * 3
  
  -- act on chunks 
  tBinaryString = char 1 to subLengthLim of aBinaryString
  delete char 1 to subLengthLim of aBinaryString
  
  vCompleteOutput = ""
  
  -- Store all the complete quanta...
  repeat while vIndex < vLengthWhole
    
    vByte1 = charToNum( char 1 of tBinaryString)
    vByte2 = charToNum( char 2 of tBinaryString)
    vByte3 = charToNum( char 3 of tBinaryString)
    
    put char (vByte1 / 4 + 1) of vBase64Lookup after vOutput
    put char ((vByte1 mod 4) * 16 + (vByte2 / 16) + 1) of vBase64Lookup after vOutput
    put char ((vByte2 mod 16) * 4 + (vByte3 / 64) + 1) of vBase64Lookup after vOutput
    put char ((vByte3 mod 64) + 1) of vBase64Lookup after vOutput
    
    vIndex = vIndex + 3
    
    -- Wrap at 76 characters
    if( ((vIndex-1) mod 57) = 0 ) then put crlf after vOutput
    
    delete char 1 to 3 of tBinaryString
    
    -- get the next chunk:
    if (vIndex - 1) mod subLengthLim = 0 then
      tBinaryString = char 1 to subLengthLim of aBinaryString
      delete char 1 to subLengthLim of aBinaryString
      put vOutput after vCompleteOutput
      vOutput = ""
    end if
    
  end repeat
  
  -- Store any partial quanta at the end....
  if( vTrailingBytes = 1 ) then
    
    -- Partial quanta containing a single byte...
    vByte1 = charToNum( char 1 of tBinaryString )
    put char (vByte1 / 4 + 1) of vBase64Lookup after vOutput
    put char ((vByte1 mod 4) * 16 + 1) of vBase64Lookup after vOutput
    put vTrail & vTrail after vOutput
    
  else if ( vTrailingBytes = 2 ) then
    
    -- Partial quanta containing two bytes...
    vByte1 = charToNum( char 1 of tBinaryString )
    vByte2 = charToNum( char 2 of tBinaryString )
    put char (vByte1 / 4 + 1) of vBase64Lookup after vOutput
    put char ((vByte1 mod 4) * 16 + (vByte2 / 16) + 1) of vBase64Lookup after vOutput
    put char ((vByte2 mod 16) * 4 + 1) of vBase64Lookup after vOutput
    put vTrail after vOutput
    
  end if
  
  put vOutput after vCompleteOutput
  return vCompleteOutput
end 


----------------------------------------
-- optimized by LingoGuy@sbcglobal.net   3/29/04
----------------------------------------
on Base64EncodePC (str)
  
  subLengthLim = 1000 -- tune for optimization ( 200 to 60,000 range vary effeciency by +/- 10%)
  
  subStrings = []
  Base64Lookup = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/"
  out = ""
  i=1
  strRemainder = str.length mod 3
  strLength = str.length - (strRemainder)
  
  repeat while i < strLength
    
    a = charToNum( str.char[ i ] )
    b = charToNum( str.char[ i+1 ] )
    c = charToNum( str.char[ i+2 ] )
    o1 = Base64Lookup.char[ ( bitAnd( a, 252 ) / 4 ) + 1 ]
    o2 = Base64Lookup.char[ ( bitAnd( (a*256)+b, 1008 ) / 16 ) + 1 ]
    o3 = Base64Lookup.char[ ( bitAnd( (b*256)+c, 4032 ) / 64 ) + 1 ]
    o4 = Base64Lookup.char[ ( bitAnd( c, 63 ) ) + 1]
    put o1 after out
    put o2 after out
    put o3 after out
    put o4 after out
    i = i + 3
    if( ((i-1) mod 57) = 0 ) then put numToChar(13) & numToChar(10) after out
    
    if length(out) > subLengthLim then --- cach segment in array, start new out string
      append subStrings, out
      out = empty
    end if
  end repeat
  if( strRemainder = 1 ) then
    a = charToNum( str.char[ i ] )
    o1 = Base64Lookup.char[ ( bitAnd( a, 252 ) / 4 ) + 1 ]
    o2 = Base64Lookup.char[ ( bitAnd( a*256, 1008 ) / 16 ) + 1 ]
    o3 = "="
    o4 = "="
    put o1 after out
    put o2 after out
    put o3 after out
    put o4 after out
  else if ( strRemainder = 2 ) then
    a = charToNum( str.char[ i ] )
    b = charToNum( str.char[ i+1 ] )
    o1 = Base64Lookup.char[ ( bitAnd( a, 252 ) / 4 ) + 1 ]
    o2 = Base64Lookup.char[ ( bitAnd( (a*256)+b, 1008 ) / 16 ) + 1 ]
    o3 = Base64Lookup.char[ ( bitAnd( b*256, 4032 ) / 64 ) + 1 ]
    o4 = "="
    put o1 after out
    put o2 after out
    put o3 after out
    put o4 after out
  end if
  
  bigOut = empty
  repeat with s in subStrings
    put s after bigOut
  end repeat
  put out after bigOut -- last uncached bit
  
  return bigOut
end

----------------------------------------
-- 
----------------------------------------
on Base64Decode str
  Base64Lookup=["A":0,"B":1,"C":2,"D":3,"E":4,"F":5,"G":6,"H":7,"I":8,"J":9,"K":10,"L":11,"M":12,"N":13,"O":14,"P":15,"Q":16,"R":17,"S":18,"T":19,"U":20,"V":21,"W":22,"X":23,"Y":24,"Z":25,"a":26,"b":27,"c":28,"d":29,"e":30,"f":31,"g":32,"h":33,"i":34,"j":35,"k":36,"l":37,"m":38,"n":39,"o":40,"p":41,"q":42,"r":43,"s":44,"t":45,"u":46,"v":47,"w":48,"x":49,"y":50,"z":51,"0":52,"1":53,"2":54,"3":55,"4":56,"5":57,"6":58,"7":59,"8":60,"9":61,"+":62,"/":63]
  out = ""
  quanta = []
  repeat with i = 1 to str.length
    ch = Base64Lookup.getaProp( str.char[i] )
    if not VoidP( ch ) then quanta.add( ch )
    if( quanta.count = 4 ) then
      a = numToChar( bitAnd( (quanta[1]*64)+quanta[2], 4080 ) / 16 )
      b = numToChar( bitAnd( (quanta[2]*64)+quanta[3], 1020 ) / 4 )
      c = numToChar( bitAnd( (quanta[3]*64)+quanta[4], 255 ) )
      put a after out
      put b after out
      put c after out
      
      quanta = []
    end if
  end repeat
  if( quanta.count = 3 ) then
    a = numToChar( bitAnd( (quanta[1]*64)+quanta[2], 4080 ) / 16 )
    b = numToChar( bitAnd( (quanta[2]*64)+quanta[3], 1020 ) / 4 )
    put a after out
    put b after out
  else if( quanta.count = 2 ) then
    a = numToChar( bitAnd( (quanta[1]*64)+quanta[2], 4080 ) / 16 )
    put a after out
  end if
  return out
end